import time
from .data import Message
from . import gui
from .data import log
from io import TextIOWrapper
from PySide6 import QtCore
from src.pyside6.gui_pyside6 import MainWindow


class Log_Reader(QtCore.QObject):
    new_trade_signal = QtCore.Signal(Message)
    new_out_trade_signal = QtCore.Signal(Message)

    def __init__(self, window: MainWindow):
        super().__init__()
        self.new_trade_signal.connect(window.new_trade, QtCore.Qt.QueuedConnection)

    def read_log(self, logfile: str) -> None:
        try:
            with open(logfile, 'r', encoding='utf8') as file:
                loglines = self.follow(file)
                for line in loglines:
                    message = Message.from_text(line)
                    log.debug(message)
                    if message and message.trade:
                        self.new_trade_signal.emit(message)
        except IOError:
            log.error(f'Error opening log file {logfile}.')

    def follow(self, thefile: TextIOWrapper):
        thefile.seek(0, 2)
        while True:
            line = thefile.readline()
            if not line:
                time.sleep(0.1)
                continue
            yield line
